@@ -1,3 +1,16 @@
+/*
+ *  Copyright (C) 2000-2012 D. Gilbert
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  This program outputs information provided by a SCSI MODE SENSE command.
+ *  Does 10 byte MODE SENSE commands by default, Trent Piepho added a "-6"
+ *  switch for force 6 byte mode sense commands.
+ *  This utility cannot modify mode pages. See the sdparm utility for that.
+ */
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -12,21 +25,7 @@
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
 
-/*
-*  Copyright (C) 2000-2011 D. Gilbert
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2, or (at your option)
-*  any later version.
-
-   This program outputs information provided by a SCSI MODE SENSE command.
-   Does 10 byte MODE SENSE commands by default, Trent Piepho added a "-6"
-   switch for force 6 byte mode sense commands.
-   This utility cannot modify mode pages. See the sdparm utility for that.
-
-*/
-
-static char * version_str = "1.37 20111113";
+static char * version_str = "1.39 20120920";
 
 #define DEF_ALLOC_LEN (1024 * 4)
 #define DEF_6_ALLOC_LEN 252
@@ -539,7 +538,7 @@ static struct page_code_desc pc_desc_disk[] = {
     {0xb, 0x0, "Medium types supported (obsolete)"},
     {0xc, 0x0, "Notch and partition (obsolete)"},
     {0xd, 0x0, "Power condition (obsolete, moved to 0x1a)"},
-    {0x10, 0x0, "XOR control"},
+    {0x10, 0x0, "XOR control"}, /* obsolete in sbc3r32 */
     {0x1a, 0xf1, "ATA Power condition"},
     {0x1c, 0x1, "Background control"},
     {0x1c, 0x2, "Logical block provisioning"},
